#!/bin/bash

#Simple script to control the dummy PLCC84 chip that goes into the Hawaii's socket.
#Copyright Richard Neill, July 2012, Free Software released under the GNU GPL3+.

#--- BEGIN CONFIGURATION ---

LEDS_OFF=0x0000				#Red LEDs are on the 4 "Pixel" clock bits, 0,1,2,3
LEDS_RED=0x000f				#Green LED is on the "Reset" clock bit, 12
LEDS_GREEN=0x1000
LEDS_ALL=$((LEDS_RED|LEDS_GREEN))
LED_CMD=pb_init				#Use pb_init to just set the bits.

CALIBRATE_CMD="ni4462_test -S"		#Self-calibrate the NI4462.
MEASURE_CMD="ni4462_test -c all -v 3 -n 200000 -f 200000 -i dc -m diff -t now -bB /dev/null" #Measure 1 second, mean and std-dev on all channels, 3V range.
NOISE_THRESH=100			#If stddev on any channel exceeds 100uV, warn.
R_PULLUP_K=10				#Inside copper box, each channel has 10k pullup to Analog +5V.
R_PLAT_NOM_K=1				#Platinum resistor sensor nominally 1k.

#--- END CONFIGURATION -----

if [ $# == 0 -o "$1" == "-h" -o "$1" == "--help" -o "$1" == help ] ;then
	HELP=true

elif [ "$1" == leds ] ;then
	LEDS=$2
	if [ "$2" == off ] ;then
		BITS=$LEDS_OFF
	elif [ "$2" == red ]; then
		BITS=$LEDS_RED
	elif [ "$2" == green ]; then
		BITS=$LEDS_GREEN
	elif [ "$2" == all ]; then
		BITS=$LEDS_ALL
	else
		HELP=true
	fi

elif [ "$1" == calibrate ] ;then
	CALIBRATE=true

elif [ "$1" == measure ] ;then
	if [ "$2" == light ] ;then
		MEASURE=light
	elif [ "$2" == temp ] ;then
		MEASURE=temperature
	else
		HELP=true
	fi
else
	echo "Invalid arguments. Use -h for help."
fi

if [ -n "$HELP" ] ; then
	cat <<-EOT
	This controls a "dummy Hawaii chip" in the dewar. Instead of the actual Rockwell Hawaii sensor, there is
	a PLCC-84 package, on which is mounted 4 red LEDs (controlled by the Pixel bits) and one high-intensity
	green LED (controlled by the Reset bit). There is also a Phototransistor, and a 1k Platinum resistance
	temperature sensor. The primary purpose is to shine light *out* from the dewar, to assist in aligning
	the optics. The platinum thermometer and phototransistor are just extras.

	All 4 of the NI4462 channels have their 'cold' (inverting) inputs connected to VRst/2. Quad_0 and 1 have
	their non-inverting inputs connected to (respectively) a Pt 1k temp sensor, and a phototransistor; these
	are pulled up by 10k to Analog +5V, and the sensors are grounded to Analog Gnd. Quad_2 is floating (so it
	reads Analog +5; Quad_3 is shorted to A_Gnd. This allows for the real voltages across the two sensors to
	be accurately calculated.

	Usage:  `basename $0 .sh` ACTION
	ACTION: leds     red|green|all|off
	        measure  light|temp
	        calibrate
	        help
	EOT

	exit 1

elif [ -n "$CALIBRATE" ];then
	echo "Self-calibrating NI4462. This may take up to 3 minutes."
	$CALIBRATE_CMD
	exit $?

elif [ -n "$LEDS" ]; then
	echo "Lighting LEDs ($LEDS) inside the dewar."
	$LED_CMD $BITS 2>/dev/null
	exit $?

elif [ -n "$MEASURE" ]; then
	echo "Measuring data from $MEASURE sensor inside the dewar."
	RESULT=$($MEASURE_CMD 2>&1)
	[ $? != 0 ] && exit 1
	MEANS=( $(echo "$RESULT" | head -n1 | sed 's/#.*//' ) )		#mean (mV), inputs 0,1,2,3
	STDDEVS=( $(echo "$RESULT" | tail -n1 | sed 's/#.*//' ) )	#std-dev (uV), inputs 0,1,2,3

	for ((i=0;i<4;i++)); do						#Check noise.
		[ `echo "${STDDEVS[$i]} > $NOISE_THRESH" | bc` == 1 ] && echo "Warning: excessive noise on channel $i: ${STDDEVS[$i]} uV. (Threshold: $NOISE_THRESH uV)."
	done

	V_5=$(echo "${MEANS[2]} - ${MEANS[3]}" | bc) 			#Analog 5V value.
	V_Temp=$(echo "${MEANS[0]} - ${MEANS[3]}" | bc)			#Voltage across temp. sensor (Pt. resistor)
	V_Light=$(echo "${MEANS[1]} - ${MEANS[3]}" | bc)		#Voltage across light sensor (phototransistor).

	I_Light=$(echo "scale=3; ($V_5 - $V_Light) / $R_PULLUP_K" | bc ) 		#Current through phototransistor. (mA).
	R_Temp=$(echo  "scale=3; ($V_Temp * $R_PULLUP_K) / ($V_5 - $V_Temp)" | bc ) 	#Value of platinum resistor (kOhm)
	R_Temp_Rel=$(echo "scale=3; $R_Temp / $R_PLAT_NOM_K" | bc )			#Value relative to nominal.

	#Get temperature. From datasheet, Pt has a relative resistance of 0.185 at -200 degC, (compared to a 1.0 at the nominal temperature of 0 degrees C).
	#Approximate (incorrectly) that this is linear. so: Temp_C ~ 245.4 * ((R/Rnominal) - 1).
	Temp_C=$(echo "scale=3; 245.4 * ($R_Temp_Rel - 1)" | bc)
	Temp_K=$(echo "$Temp_C + 273.15" | bc)

	if [ $MEASURE == light ] ;then
		echo "Phototransistor voltage is: $V_Light V, with a supply voltage of $V_5 V via $R_PULLUP_K kOhm."
		echo "Current is: $I_Light mA."
	else
		echo "Platinum resistor voltage is $V_Temp V, with a supply voltage of $V_5 V via $R_PULLUP_K kOhm."
		echo "Platinum resistor resistance is $R_Temp kOhm; a factor $R_Temp_Rel of the nominal $R_PLAT_NOM_K kOhm."
		echo "Approximate Temperature is $Temp_K K ($Temp_C degC)."
	fi
	exit 0

fi
