#!/bin/bash

#Measure temperature inside dewar.
#Copyright Richard Neill July 2012; this is free software under GNU GPLv3+

#--- BEGIN CONFIGURATION ---

LOOPBACK_CMD="filterctl -k"		#Loopback command.
DEWAR_SLEEP_CMD="filterctl -z"		#Powerdown dewar, end loopback.
CALIBRATE_CMD="ni4462_test -S"		#Self-calibrate the NI4462.
MEASURE_CMD="ni4462_test -c 2 -v 3 -n 200000 -f 200000 -i dc -m diff -t now -g -bB /dev/null" #Measure 1 second, mean and std-dev on channel 2, 3V range.
NOISE_THRESH=100			#If stddev on this channel exceeds 100uV, warn.
CONVERT_CMD="cy7sd7_volt2temp"		#Convert from volts to temp, using curve for this sensor.
STABILISE_TIME_S=1			#How long to let the power-supply voltage and op-amp output filter stabilise.
CALIBRATION_ERR_ROOMTEMP=1.65		#Offset - measured against thermometer at room temp. 

#--- END CONFIGURATION -----

#Parse args.
if [ "$1" == "-h" -o "$1" == "--help" -o "$1" == help ] ;then
	HELP=true
elif [ "$1" == -s ] ;then
	SELFCAL=true
elif [ $# != 0 ]; then
	echo "Invalid arguments. Use -h for help."
	exit 1
fi

#Check for binaries.
if ! which ${CALIBRATE_CMD% *} >/dev/null 2>&1 ; then
	echo "Missing program: ${CALIBRATE_CMD% *}"; exit 1
elif ! which ${CONVERT_CMD% *} >/dev/null 2>&1 ; then
	echo "Missing program: ${CONVERT_CMD% *}"; exit 1
elif ! which ${LOOPBACK_CMD% *} >/dev/null 2>&1 ; then
	echo "Missing program: ${LOOPBACK_CMD% *}"; exit 1
fi

#Help
if [ -n "$HELP" ] ; then
	cat <<-EOT
	This measures the temperature near the Hawaii chip in the dewar.
	The dewar contains a temperature-sensing diode, mounted on the same PLCC-socket that carries the Hawaii
	sensor. The diode is the Omega CY7-SD7, specified within 1.5%, over the range 10-475 Kelvin, at 10uA. It is
	electronically connected via the NI4462 ADC's channel 2 [numbering starts at 0]; this channel is usually
	connected to the Hawaii's quadrant 3, but in loopback mode [see filterctl], it is swapped over by a
	relay, and power is supplied to the driver/buffer circuit. Measurement takes about 2 seconds.
	Note that, in loopback mode, quadrants 3 and 4 are temporarily unavailable for light-detection.
	Aside: Liquid Nitrogen is 77 Kelvin (-196 degC);  Lab is about 293 K (20 degC).
	Calibration: at room-temperature, the result is (experimentally) about $CALIBRATION_ERR_ROOMTEMP degC too low.

	USAGE:    `basename $0 .sh` [OPTIONS]
	OPTIONS:  -h          show this help
	          -s          first, do self-cal of NI4462 (takes about 3 minutes).
	          [none]      measure the temperature.

	EOT
	exit 0
fi

#Optionally, self-cal.
if [ -n "$SELFCAL" ] ;then
        echo "Self-calibrating NI4462. This may take up to 3 minutes."
        if ! $CALIBRATE_CMD >/dev/null 2>&1 ; then
		echo "Error doing self-calibration."; exit 1
	fi

fi

#Engage loopback mode. Wait 1 second for power to stabilise for the 10uA supply. If this fails, it (probably) means that dewar-power is unavailable.
if ! $LOOPBACK_CMD >/dev/null 2>&1; then
	echo "Error running $LOOPBACK_CMD"; exit 1
fi
sleep $STABILISE_TIME_S

#Measure for 1 second, get mean and stddev.
RESULT=$($MEASURE_CMD 2>&1)
[ $? != 0 ] && exit 1
MEAN=($(echo "$RESULT" | head -n1 | sed 's/#.*//' )) 	#mean (mV),
STDDEV=($(echo "$RESULT" | tail -n1 | sed 's/#.*//' ))	#std-dev (uV)

#Check for excess noise.
if [ `echo "$STDDEV > $NOISE_THRESH" | bc` == 1 ]; then
	echo "WARNING: excessive noise: $STDDEV uV. (Threshold: $NOISE_THRESH uV)." >&2
fi

#Scale to volts:
MEAN_VOLTS=$(echo "scale=6; ($MEAN / 1000)" | bc | awk '{printf "%f", $0}' )
STDDEV_VOLTS=$(echo "scale=6; ($STDDEV / 1000000)" | bc | awk '{printf "%f", $0}' )

#Convert voltage-reading to temperature.
TEMP_K=$($CONVERT_CMD $MEAN_VOLTS);
if [ $? != 0 ] ;then
	echo "Error: couldn't convert voltage $MEAN_VOLTS V to a valid temperature."
	exit 1
fi
TEMP_C=$(echo "$TEMP_K - 273.15" | bc)

#Disengage loopback mode.
if ! $DEWAR_SLEEP_CMD >/dev/null 2>&1; then
	echo "Error running $DEWAR_SLEEP_CMD"; exit 1
fi

#Results
echo "CY7-SD7 diode voltage:  $MEAN_VOLTS +/- $STDDEV_VOLTS V"
echo "Temperature (K):        $TEMP_K"
echo "Temperature (degC):     $TEMP_C"
exit 0
